<?php
namespace GopNext\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class OperatingBudgetConclusionService {
    
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
  
    /**
    * Start transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
  
    /**
    * Commit transaction on database server
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function commit() {
        $this->connection->commit();
    }
  
    /**
    * Roll back transaction in database
    *
    * @access public
    * @author Jefferson Gouvea
    */
    public function rollback() {
        $this->connection->rollback();
    }

    public function completeBudget($data, $nrLista) {
        try {
            $params = [
                "NRLISTA"       => $nrLista,
                "IDTIPOORCAOP"  => $data["IDTIPOORCAOP"],
                "NRANOORCAGOP"  => $data["NRANOORCAGOP"],
                "IDPERIODOORCA" => $data["IDPERIODOORCA"],
                "CDOPERADOR"    => $this->environment->getCdOperador(),
            ];
            
            $script = "BEGIN
                          FOR C_ORCAMENTO IN (
                             SELECT O.NRORCAMENTOGOP, I.IDTIPOORCAOP, O.CDFILIAL, C.NRCONCPREORCAGOP
                               FROM ORCAMENTOGOP O INNER JOIN LISTA L ON L.CDLISTA = O.CDFILIAL
                                                   INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                           AND I.IDTIPOORCAOP = :IDTIPOORCAOP
                                                   LEFT  JOIN CONCPREORCAGOP C ON C.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                              AND C.IDTIPOORCAOP = I.IDTIPOORCAOP
                                                                              AND C.CDFILIAL = O.CDFILIAL
                              WHERE L.NRLISTA = :NRLISTA
                                AND O.NRANOORCAGOP = :NRANOORCAGOP
                                AND O.IDPERIODOORCA = :IDPERIODOORCA) LOOP
      
                             IF C_ORCAMENTO.NRCONCPREORCAGOP IS NULL THEN
                                INSERT INTO CONCPREORCAGOP (NRCONCPREORCAGOP, NRORCAMENTOGOP, CDFILIAL, IDTIPOORCAOP,
                                                            DSOBSERVACAO, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO,
                                                            CDOPERULTATU, IDSTATUS)
                                     VALUES (SEQ_CONCPREORCAGOP.NEXTVAL, C_ORCAMENTO.NRORCAMENTOGOP, C_ORCAMENTO.CDFILIAL, C_ORCAMENTO.IDTIPOORCAOP,
                                             NULL, SYSDATE, SYSDATE, :CDOPERADOR,
                                             :CDOPERADOR, 'CONCLUIDO');
                             ELSE
                                UPDATE CONCPREORCAGOP
                                   SET IDSTATUS = 'CONCLUIDO',
                                       CDOPERULTATU = :CDOPERADOR,
                                       DTULTATU = SYSDATE
                                 WHERE NRCONCPREORCAGOP = C_ORCAMENTO.NRCONCPREORCAGOP;
                             END IF;
                          END LOOP;      
                       END;";
            
            $stmt = $this->connection->prepare($script);
            $stmt->execute($params);
        } catch (Exception $ex) {
            throw $ex;
        }
    }
}