import { Metadata, Http, Loader } from '@zeedhi/core';
import {
  Modal,
  ModalService,
  Button,
  Text,
  Textarea,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';

export class ApprovalBudgetModalController {
  private approvalBudgetModal!: Modal;

  private selectedItems;
  
  private modalButton;
  
  private modalApprovalDescription;
  
  public async createApprovalBudgetModal(nameComponent, selectedItems) {
    this.approvalBudgetModal = await ModalService.createFromJson('/modals/approvalBudgetModal', true);
    await this.approvalBudgetModal.show();
    this.selectedItems = selectedItems;
    this.modalButtonRules(nameComponent);
  }

  private async modalButtonRules(nameComponent) {
    const modalApprovalTitle = Metadata.getInstance<Text>('modalBudgetTitle');
    this.modalApprovalDescription = await Metadata.getInstance<Textarea>('modalBudgetDescription');
    
    if (nameComponent === 'approveButtonOperational') {
      this.modalButton = Metadata.getInstance<Button>('approveBudgetButton');
      modalApprovalTitle.text = 'Aprovar';
    } else if (nameComponent === 'reproveButtonOperational') {
      this.modalButton = Metadata.getInstance<Button>('reproveBudgetButton');
      modalApprovalTitle.text = 'Reprovar';
    } else if (nameComponent === 'cancelButtonOperational') {
      this.modalButton = Metadata.getInstance<Button>('cancelBudgetButton');
      modalApprovalTitle.text = 'Cancelar Aprovação';
    }
    this.modalButton.isVisible = true;
  }
  
  public async updateBudgetStatus(data) {
    const operationalBudgetApprovalController = Loader.getInstance('OperationalBudgetApprovalController');
    const utilities = new Utilities();
    
    Http.post('/updateBudgetStatus', {
      requestType: 'Row',
      row: data,
    }).then(() => {
      utilities.hideDialogue();
      operationalBudgetApprovalController.reloadGrid();
    });
  }
  
  public async showModalConfirmation({ component }) {
    const utilities = new Utilities();
    let dialogMessage;
    let activityType;
    if (component.name === 'approveBudgetButton') {
      dialogMessage = 'gop::CONFIRM_APPROVE_BUDGET';
      activityType = 'Approve';
    } else if (component.name === 'reproveBudgetButton') {
      dialogMessage = 'gop::CONFIRM_REPROVE_BUDGET';
      activityType = 'Reprove';
    } else {
      dialogMessage = 'gop::CONFIRM_CANCEL_BUDGET';
      activityType = 'Cancel';
    }
    const dataForm = {
      ACTIVITYTYPE: activityType,
      MESSAGE: dialogMessage,
      SELECTEDITEMS: this.selectedItems,
      DESCRIPTIONMODAL: this.modalApprovalDescription.internalValue,
    };
    utilities.showDialog(this.updateBudgetStatus, dataForm, 'YN');
    this.destroyBudgetModal();
  }
  
  public rulesToApply() {
    const instanceDescription = this.modalApprovalDescription.value;
    const withoutSpaces = instanceDescription.trim();
    if (instanceDescription === null || instanceDescription.length < 5 || withoutSpaces.length === 0) {
      this.modalButton.disabled = true;
    } else {
      this.modalButton.disabled = false;
    }
  }
  
  public destroyBudgetModal() {
    this.approvalBudgetModal?.destroy();
  }

  public closeApprovalBudgetModal() {
    this.approvalBudgetModal.hide();
  }
}
