import {
  Metadata,
  IEventParam,
  I18n,
  Http,
} from '@zeedhi/core';
import {
  Modal,
  ModalService,
  SelectMultiple,
  Button,
  LoadingService,
  Number as NumberInput,
  Number as ZdNumber,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';
import { ZdTabs, ZdForm, ZdCol } from '@zeedhi/vuetify';
import { Utilities } from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class BreakdownOfDetailBudgetModalController {
  private breakdownOfDetailBudgetModal!: Modal;
  
  public clean = false;
  
  public isEditing = true;
  
  private dataDiners = [];
  
  public title: string = ' ';
  
  private backupValue;
  
  private nrItDdetalOrcaMesGOP;
  
  private dietPad;
  
  public async createBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal = await ModalService.createFromJson('/modals/breakdownOfDetailBudgetModal', true);
    this.breakdownOfDetailBudgetModal.show();
  }
  
  public destroyBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal?.destroy();
  }
  
  public async openBreakdownOfDetailBudgetModal({ row }: IEventParam<Button>) {
    if (!this.breakdownOfDetailBudgetModal) {
      await this.createBreakdownOfDetailBudgetModal();
    } else {
      await this.breakdownOfDetailBudgetModal.show();
    }
    this.serviceDetails(row.NRITDETALORCAMESGOP);
  }
  
  public closeBreakdownOfDetailBudgetModal() {
    this.breakdownOfDetailBudgetModal.hide();
  }
  
  public defineTitle() {
    if (this.dataDiners) {
      this.title = I18n.translate('Unidade').concat(
        ': ',
        this.dataDiners[0].CDFILIAL,
        ' - ',
        this.dataDiners[0].NMFILIAL,
        ' | ',
        this.dataDiners[0].MESANO,
      );
    } else {
      this.title = ' ';
    }
  }
  
  /**
  * Triggers the function that defines the pages as not loaded, and leaves the first visible page active.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public clearData() {
    const tabsBudget = Metadata.getInstance<ZdTabs>('tabsBudget');
    BudgetParameters.setNotRecharged();
    tabsBudget.activeTab = tabsBudget.tabs.findIndex((obj) => (obj.isVisible === true));
  }
  
  /**
  * Updates the search for units in the filter according to the fields entered.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public createDataSourceFilter() {
    const filial = Metadata.getInstance<SelectMultiple>('CDFILIAL');
    const grupFili = Metadata.getInstance<SelectMultiple>('CDGRUPFILI');
    const areaGere = Metadata.getInstance<SelectMultiple>('CDAREAGERE');
    const segMerc = Metadata.getInstance<SelectMultiple>('CDSEGMER');
    
    filial.datasource.setFilter({
      CDGRUPFILI: grupFili.value && grupFili.value.length ? grupFili.value.join('_') : 'T',
      CDAREAGERE: areaGere.value && areaGere.value.length ? areaGere.value.join('_') : 'T',
      CDSEGMER: segMerc.value && segMerc.value.length ? segMerc.value.join('_') : 'T',
    });
  }
  
  public async serviceDetails(nrItDdetalOrcaMesGOP) {
    const routeName = 'gop10000_serviceDetails';
    const utilities = new Utilities();
    const sendObject = {
      filter: [{
        name: 'NRITDETALORCAMESGOP',
        operator: '=',
        value: nrItDdetalOrcaMesGOP,
      }],
    };
    this.nrItDdetalOrcaMesGOP = nrItDdetalOrcaMesGOP;
    const gridDiet = Metadata.getInstance<TekGrid>('gridDiet');
    const colDiet = Metadata.getInstance<ZdCol>('colDiet');
    const colOperationalItem = Metadata.getInstance<ZdCol>('colOperationalItem');
    
    LoadingService.show();
    try {
      const response = await Http.post(`/${routeName}`, sendObject);
      const data = response.data.dataset[routeName];
      const dataDiet = response.data.dataset[routeName.concat('Diet')];
        
      this.dataDiners = data;
      
      if (data && data[0] && dataDiet && data[0].IDTIPOSERV === 'D') {
        const curDiet = data.filter((diet) => (diet.CDDIETPAD === dataDiet[0].CDDIETPAD));
        
        await gridDiet.cancelEditedRows();
        await gridDiet.datasource.updateData([]);
        await gridDiet.datasource.updateData(dataDiet);
        
        colDiet.cols = 3;
        colOperationalItem.cols = 9;
        
        gridDiet.datasource.currentRow = dataDiet[0];
        
        this.dietPad = dataDiet[0].CDDIETPAD;
        this.updateForm(curDiet[0]);
      } else {
        await gridDiet.cancelEditedRows();
        await gridDiet.datasource.updateData([]);
        
        colDiet.cols = 0;
        colOperationalItem.cols = 12;
      }
      this.defineTitle();
      
      LoadingService.hide();
    } catch (e) {
      utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
      LoadingService.hide();
      console.warn(e);
    }
  }
  
  public updateForm(data) {
    const formDiners = Metadata.getInstance<ZdForm>('gridFormDetServ');
    const totalService = Metadata.getInstance<NumberInput>('TOTSERV');
    
    if (data) {
      formDiners.value.QTSEG = data.QTSEG;
      formDiners.value.QTTER = data.QTTER;
      formDiners.value.QTQUA = data.QTQUA;
      formDiners.value.QTQUI = data.QTQUI;
      formDiners.value.QTSEX = data.QTSEX;
      formDiners.value.QTSAB = data.QTSAB;
      formDiners.value.QTDOM = data.QTDOM;
      formDiners.value.VRCUSTPADR = data.VRCUSTPADR;
      
      this.backupValue = {
        QTSEG: data.QTSEG,
        QTTER: data.QTTER,
        QTQUA: data.QTQUA,
        QTQUI: data.QTQUI,
        QTSEX: data.QTSEX,
        QTSAB: data.QTSAB,
        QTDOM: data.QTDOM,
        VRCUSTPADR: data.VRCUSTPADR,
      };
    }
    
    formDiners.value.TOTSEG = formDiners.value.QTSEG * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTTER = formDiners.value.QTTER * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTQUA = formDiners.value.QTQUA * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTQUI = formDiners.value.QTQUI * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTSEX = formDiners.value.QTSEX * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTSAB = formDiners.value.QTSAB * formDiners.value.VRCUSTPADR;
    formDiners.value.TOTDOM = formDiners.value.QTDOM * formDiners.value.VRCUSTPADR;
    
    totalService.value = formDiners.value.TOTSEG + formDiners.value.TOTTER
      + formDiners.value.TOTQUA + formDiners.value.TOTQUI
      + formDiners.value.TOTSEX + formDiners.value.TOTSAB
      + formDiners.value.TOTDOM;
  }
  
  public dietRowClick({ row }: any) {
    const curDiet = this.dataDiners.filter((diet) => (diet.CDDIETPAD === row.CDDIETPAD));
    
    this.dietPad = curDiet[0].CDDIETPAD;
    this.updateForm(curDiet[0]);
  }
  
  public cancelChanges() {
    const formDiners = Metadata.getInstance<ZdForm>('gridFormDetServ');
    
    formDiners.value.QTSEG = this.backupValue.QTSEG;
    formDiners.value.QTTER = this.backupValue.QTTER;
    formDiners.value.QTQUA = this.backupValue.QTQUA;
    formDiners.value.QTQUI = this.backupValue.QTQUI;
    formDiners.value.QTSEX = this.backupValue.QTSEX;
    formDiners.value.QTSAB = this.backupValue.QTSAB;
    formDiners.value.QTDOM = this.backupValue.QTDOM;
    formDiners.value.VRCUSTPADR = this.backupValue.VRCUSTPADR;
    
    this.updateForm(null);
    
    this.isEditing = true;
    // this.disableClear = !this.isEditing;
  }
  
  public changeValue() {
    this.updateForm(null);
    this.isEditing = false;
    // this.disableClear = !this.isEditing;
  }
  
  public async importServiceSpecif({ component }) {
    const utilities = new Utilities();
    const action = component.name.replace(/^btnImport/, '');
    
    const data = {
      NRITDETALORCAMESGOP: this.nrItDdetalOrcaMesGOP,
      IMPORT: action,
      MESSAGE: '',
    };
    
    if (action === 'Cust') {
      data.MESSAGE = I18n.translate('gop::CUST_SERVIMP');
    } else if (action === 'Diners') {
      data.MESSAGE = I18n.translate('gop::COMENS_SERVIMP');
    } else {
      data.MESSAGE = '';
    }
    
    await utilities.showDialog(this.importService, data, 'YN');
  }
  
  public importService = async (data) => {
    const routeName = 'gop10000_importService';
    const utilities = new Utilities();
    
    LoadingService.show();
    try {
      const response = await Http.post(`/${routeName}`, data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      utilities.successMessage();
      await this.serviceDetails(this.nrItDdetalOrcaMesGOP);
      LoadingService.hide();
      utilities.hideDialogue();
    } catch (e) {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    }
  };
  
  public async clearServices() {
    const utilities = new Utilities();
    const data = {
      NRITDETALORCAMESGOP: this.nrItDdetalOrcaMesGOP,
      MESSAGE: I18n.translate('gop::REMOVE_SERVICE'),
    };
    
    await utilities.showDialog(this.cleanService, data, 'YN');
  }
  
  public cleanService = async (data) => {
    const routeName = 'gop10000_cleanService';
    const utilities = new Utilities();
    
    LoadingService.show();
    try {
      const response = await Http.post(`/${routeName}`, data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      utilities.successMessage();
      await this.serviceDetails(this.nrItDdetalOrcaMesGOP);
      LoadingService.hide();
      utilities.hideDialogue();
    } catch (e) {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    }
  };
  
  public async saveChanges() {
    const utilities = new Utilities();
    const routeName = 'gop10000_saveChangeServiceDetail';
    const formDiners = Metadata.getInstance<ZdForm>('gridFormDetServ');
    
    /*
    let quantityValue = 0;
    let totalValue = 0;
    let price = 0;
    */
    
    /*
    const gridDetail = Metadata.getInstance<TekGrid>('gop10000_gridInvoicingDetail');
    const { uniqueKey } = gridDetail.datasource;
    const rowKey = (gridDetail.datasource as any).currentRow[uniqueKey];
    const totalOldValue = (gridDetail.datasource as any).currentRow.VRORCAMENTO;
    */
    
    /*
    quantityValue = formDiners.value.QTSEG + formDiners.value.QTTER
      + formDiners.value.QTQUA + formDiners.value.QTQUI
      + formDiners.value.QTSEX + formDiners.value.QTSAB
      + formDiners.value.QTDOM;
    
    price = formDiners.value.VRCUSTPADR;
      
    totalValue = quantityValue * price;
    */
    
    /*
    gridDetail.editing = true;
    gridDetail.updateCell(rowKey, 'QTORCAMENTO', quantityValue);
    gridDetail.updateCell(rowKey, 'VRPRECOBASE', price);
    gridDetail.updateCell(rowKey, 'VRORCAMENTO', totalValue);
    gridDetail.editing = false;
    */
    
    const data = {
      filter: [{
        name: 'NRITDETALORCAMESGOP',
        operator: '=',
        value: this.nrItDdetalOrcaMesGOP,
      },
      {
        name: 'CDDIETPAD',
        operator: '=',
        value: this.dietPad,
      },
      {
        name: 'DATA',
        operator: '=',
        value: formDiners.value,
      }],
    };
    
    LoadingService.show();
    try {
      Http.post(`/${routeName}`, {
        requestType: 'DataSet',
        dataset: data,
      });
      
      LoadingService.hide();
      utilities.successMessage();
      
      this.isEditing = true;
      // this.disableClear = !this.isEditing;
      
      this.updateDetailGrid('gop10000_gridInvoicingDetail');
    } catch (e) {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    }
  }
  
  public async updateDetailGrid(type) {
    const gridDetail = Metadata.getInstance<TekGrid>(type);
    const gridMaster = Metadata.getInstance<TekGrid>(gridDetail.name.replace(/Detail$/, ''));
    // const { uniqueKey } = gridDetail.datasource;
    // const rowKey = (gridDetail.datasource as any).currentRow[uniqueKey];
    const currentRowKey = (gridDetail.datasource as any).currentRow.NRITDETALORCAMESGOP;
    const currentMasterRowKey = (gridMaster.datasource as any).currentRow.NRORCAMENTOGOP;
    const columnMaster = gridMaster.getColumn(gridMaster.currentColumn.name);
    
    let quantityValue = 0;
    let totalValue = 0;
    let price = 0;
    let differenceValue = 0;
    
    const data = (gridDetail.datasource as any).allData;
    const dataRow = data.find((obj) => (obj.NRITDETALORCAMESGOP === currentRowKey));
    
    const dataMaster = (gridMaster.datasource as any).allData;
    const dataMasterRow = dataMaster.find((obj) => (obj.NRORCAMENTOGOP === currentMasterRowKey));
    
    if (type === 'gop10000_gridInvoicingDetail') {
      const formDiners = Metadata.getInstance<ZdForm>('gridFormDetServ');
      
      quantityValue = formDiners.value.QTSEG + formDiners.value.QTTER
        + formDiners.value.QTQUA + formDiners.value.QTQUI
        + formDiners.value.QTSEX + formDiners.value.QTSAB
        + formDiners.value.QTDOM;
      
      price = formDiners.value.VRCUSTPADR;
        
      totalValue = quantityValue * price;
    }
    dataRow.QTORCAMENTO = quantityValue;
    dataRow.VRPRECOBASE = price;
    differenceValue = totalValue - dataRow.VRORCAMENTO;
    dataRow.VRORCAMENTO = totalValue;
    
    await gridDetail.datasource.updateData([]);
    await gridDetail.datasource.updateData(data);
    gridDetail.datasource.currentRow = data.find((obj) => (obj.NRITDETALORCAMESGOP === currentRowKey));
    
    const invoicingDetail = Metadata.getInstance<ZdNumber>('invoicingDetailValue');
    invoicingDetail.value += differenceValue;
    
    dataMasterRow[columnMaster.name] = invoicingDetail.value;
    
    await gridMaster.datasource.updateData([]);
    await gridMaster.datasource.updateData(dataMaster);
    gridMaster.datasource.currentRow = dataMaster.find((obj) => (obj.NRORCAMENTOGOP === currentMasterRowKey));
    (gridMaster as any).currentColumn = columnMaster;
  }
  
  get disableClear() {
    return !this.isEditing;
  }
}
