import {
  Metadata,
  // IEventParam,
  I18n,
  Loader,
  Http,
} from '@zeedhi/core';
import {
  Modal,
  ModalService,
  // SelectMultiple,
  // Button,
  LoadingService,
  SelectMultiple,
  Form,
  // Number as NumberInput,
  // Number as ZdNumber,
} from '@zeedhi/common';
import {
  ZdButton,
  ZdTooltip,
  ZdText,
} from '@zeedhi/vuetify';
// import BudgetParameters from '@/controllers/BudgetParameters';
/*
import {
  ZdTabs,
  // ZdForm,
  // ZdCol,
} from '@zeedhi/vuetify';
*/
import { Utilities } from '@/util/Utilities';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class DefinitionOfBudgetCalcModalController {
  private definitionOfBudgetCalcModal!: Modal;
  
  private dataBudget;
  
  private mapping: { [key: string]: string } = {
    INVESTIMENTO: 'gop10000_gridInvestment',
    IMPOSTOSOBLUCRO: 'gop10000_gridProfitTax',
    IMPOSTOARECUPERAR: 'gop10000_gridTaxRecovered',
    IMPOSTO: 'gop10000_gridTurnoverTax',
    DESPENTRADA: 'gop10000_gridEntryRelease',
    DESPDIVERSA: 'gop10000_gridMiscellaneous',
    DESPMATRIZ: 'gop10000_gridHeadquarters',
    MAOOBRA: 'gop10000_gridLabor',
    CONSUMO: 'gop10000_gridConsumption',
    FATURAMENTO: 'gop10000_gridInvoicing',
    TAXAADM: 'gop10000_gridAdministrateTax',
    DEPRECIACAO: 'gop10000_gridDepreciation',
  };
  
  public async createDefinitionOfBudgetCalcModal() {
    this.definitionOfBudgetCalcModal = await ModalService.createFromJson('/modals/definitionOfBudgetCalcModal', true);
    this.definitionOfBudgetCalcModal.show();
  }
  
  public destroyDefinitionOfBudgetCalcModal() {
    this.definitionOfBudgetCalcModal?.destroy();
    this.definitionOfBudgetCalcModal = undefined;
  }
  
  public async openDefinitionOfBudgetCalcModal(data) {
    if (!this.definitionOfBudgetCalcModal) {
      await this.createDefinitionOfBudgetCalcModal();
    } else {
      await this.definitionOfBudgetCalcModal.show();
    }
    this.dataBudget = data;
    this.visualizationModal();
  }
  
  public closeDefinitionOfBudgetCalcModal() {
    this.definitionOfBudgetCalcModal.hide();
  }
  
  /**
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async adjustBudget() {
    const formBudgetAdjustments = Metadata.getInstance<Form>('formBudgetAdjustments');
    
    if (formBudgetAdjustments.value.selectMonths
      && formBudgetAdjustments.value.selectOperation
      && formBudgetAdjustments.value.valueBudgetAdjustments) {
      const utilities = new Utilities();
      
      this.dataBudget.MONTH = formBudgetAdjustments.value.selectMonths;
      this.dataBudget.OPERATION = formBudgetAdjustments.value.selectOperation;
      this.dataBudget.VALUE = formBudgetAdjustments.value.valueBudgetAdjustments;
      this.dataBudget.ITEM = formBudgetAdjustments.value.selectItems;
      this.dataBudget.DETAL = formBudgetAdjustments.value.IDSEMDETALHAMENTO;
      this.dataBudget.MESSAGE = I18n.translate('gop::CALC_AJUSTE');
      
      const confirmed = await utilities.showDialog(this.adjustmentsBudget, this.dataBudget, 'YN');
      if (confirmed === 'Yes') {
        const mainBudget = Loader.getInstance('CalculationOperatingBudgetController');
        const grid = Metadata.getInstance<TekGrid>(this.mapping[this.dataBudget.IDTIPOORCAOP]);
        
        await mainBudget.rechargeBudget({ component: grid });
        await grid.selectAll(false);
        mainBudget.checkLine = false;
        
        this.destroyDefinitionOfBudgetCalcModal();
      }
    } else {
      formBudgetAdjustments.validate();
    }
  }
  
  public async adjustmentsBudget(data): Promise<void> {
    const utilities = new Utilities();
    
    try {
      LoadingService.show();
      
      const response = await Http.post('/adjustmentsBudget', data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      utilities.successMessage();
      utilities.hideDialogue();
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    } finally {
      LoadingService.hide();
    }
  }
  
  public toggleCollapseCard({ component }) {
    const otherCollapseCard = component.name === 'collapseBudgetAdjustments'
      ? 'collapseAutomaticCalc' : 'collapseBudgetAdjustments';
    const collapse = (component.parent as any).getChildInstance(otherCollapseCard);
    
    collapse.expanded = !collapse.expanded;
  }
  
  public visualizationModal() {
    const btnCalcBudget = Metadata.getInstance<ZdButton>('btnCalcBudget');
    const infoCalcBudget = Metadata.getInstance<ZdButton>('infoCalcBudget');
    const tooltipCalcBudget = Metadata.getInstance<ZdTooltip>('tooltipCalcBudget');
    
    const btnDROBudget = Metadata.getInstance<ZdButton>('btnDROBudget');
    const infoDROBudget = Metadata.getInstance<ZdButton>('infoDROBudget');
    
    if (this.dataBudget.IDTIPOORCAOP === 'DEPRECIACAO') {
      btnCalcBudget.isVisible = true;
      infoCalcBudget.isVisible = true;
      btnDROBudget.isVisible = false;
      infoDROBudget.isVisible = false;
      
      btnCalcBudget.label = I18n.translate('gop::CALC_ATIVIMOBGPA');
      tooltipCalcBudget.label = I18n.translate('gop::CALC_DESCATIVIMOBGPA');
    } else {
      btnCalcBudget.isVisible = false;
      infoCalcBudget.isVisible = false;
      btnDROBudget.isVisible = true;
      infoDROBudget.isVisible = true;
    }
    
    this.configBudgetAdjustment();
  }
  
  public async calculationBudget(data): Promise<void> {
    const utilities = new Utilities();
    
    try {
      LoadingService.show();
      
      const response = await Http.post('/budgetCalculation', data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      utilities.successMessage();
      utilities.hideDialogue();
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    } finally {
      LoadingService.hide();
    }
  }
  
  /*
    CALCULATION:
       C => Standard calculation
       R => Based on the result
       A => Previous budget
  */
  public async standardBudgetCalculation({ component }) {
    const utilities = new Utilities();
    let buttonPressed = 'C';
    let mainGrid = '';
    
    if (component.name === 'btnDuplicateBudget') {
      buttonPressed = 'A';
      this.dataBudget.MESSAGE = I18n.translate('gop::CALC_MENSPREV');
    } else if (component.name === 'btnDROBudget') {
      buttonPressed = 'R';
      this.dataBudget.MESSAGE = I18n.translate('gop::CALC_MENSDRO');
    }
    
    if (this.dataBudget.IDTIPOORCAOP === 'DEPRECIACAO') {
      mainGrid = 'gop10000_gridDepreciation';
      if (buttonPressed === 'C') {
        this.dataBudget.MESSAGE = I18n.translate('gop::CALC_MENSATIVIMOVGPA');
      }
      this.dataBudget.CALCULATION = buttonPressed;
    }
    
    const confirmed = await utilities.showDialog(this.calculationBudget, this.dataBudget, 'YN');
    if (confirmed === 'Yes') {
      const mainBudget = Loader.getInstance('CalculationOperatingBudgetController');
      const grid = Metadata.getInstance<TekGrid>(mainGrid);
      
      await mainBudget.rechargeBudget({ component: grid });
      await grid.selectAll(false);
      mainBudget.checkLine = false;
      
      this.destroyDefinitionOfBudgetCalcModal();
    }
  }
  
  public selectedOperation({ component }: any) {
    const selectedItem = component.value;
    const textObservation = Metadata.getInstance<ZdText>('textObservation');
    const translations = {
      1: 'gop::CALC_ATRIBUIR',
      2: 'gop::CALC_SOMAR',
      3: 'gop::CALC_SUBTRAIR',
      4: 'gop::CALC_ACRESCER',
      5: 'gop::CALC_DECRESCER',
    };
    
    if (selectedItem in translations) {
      textObservation.text = I18n.translate(translations[selectedItem]);
    }
  }
  
  public async configBudgetAdjustment() {
    const utilities = new Utilities();
    
    try {
      const sendObject = {
        filter: [{
          name: 'CDFILIAL',
          operator: '=',
          value: this.dataBudget.CDFILIAL,
        },
        {
          name: 'IDPERIODOORCA',
          operator: '=',
          value: this.dataBudget.IDPERIODOORCA,
        },
        {
          name: 'IDTIPOORCAOP',
          operator: '=',
          value: this.dataBudget.IDTIPOORCAOP,
        },
        {
          name: 'NRANOORCAGOP',
          operator: '=',
          value: this.dataBudget.NRANOORCAGOP,
        },
        {
          name: 'NRPERIODOORCA',
          operator: '=',
          value: this.dataBudget.NRPERIODOORCA,
        }],
      };
      LoadingService.show();
      
      const response = await Http.post('/configBudgetAdjustment', sendObject);
      
      const dataSourceMonth = response.data.dataset.budgetMonths.map((month) => {
        const months: string[] = [
          'Janeiro', 'Fevereiro', 'Março', 'Abril', 'Maio', 'Junho',
          'Julho', 'Agosto', 'Setembro', 'Outubro', 'Novembro', 'Dezembro',
        ];
        
        const monthIndex = parseInt(month.MES, 10) - 1;
        return {
          name: I18n.translate(months[monthIndex]),
          value: monthIndex + 1,
        };
      });
      const selectMonths = Metadata.getInstance<SelectMultiple>('selectMonths');
      selectMonths.datasource.updateData(dataSourceMonth);
      
      const dataSourceItem = response.data.dataset.detailBudget.map((item) => (
        {
          name: I18n.translate(item.IDTIPO).concat(' - ', item.NMDESCRICAO),
          value: item.IDCODIGO,
        }
      ));
      const selectItems = Metadata.getInstance<SelectMultiple>('selectItems');
      selectItems.datasource.updateData(dataSourceItem);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      utilities.successMessage();
      utilities.hideDialogue();
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    } finally {
      LoadingService.hide();
    }
  }
}
